/*********************************************************
 * Module name: ftl_api.c
 *
 * Copyright 2010, 2011. All Rights Reserved, Crane Chu.
 *
 * This file is part of OpenNFM.
 *
 * OpenNFM is free software: you can redistribute it and/or 
 * modify it under the terms of the GNU General Public 
 * License as published by the Free Software Foundation, 
 * either version 3 of the License, or (at your option) any 
 * later version.
 * 
 * OpenNFM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE. See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General Public 
 * License along with OpenNFM. If not, see 
 * <http://www.gnu.org/licenses/>.
 *
 * First written on 2010-01-01 by cranechu@gmail.com
 *
 * Module Description:
 *    FTL APIs.
 *
 *********************************************************/

#include <core\inc\cmn.h>
#include <core\inc\ftl.h>
#include <core\inc\ubi.h>
//#include <core\inc\mtd.h>
#include <sys\sys.h>
#include "ftl_inc.h"
#include <core\inc\buf.h>
#include <string.h>
/***************nc test*********/
extern int printSnap;
extern int hiddenFlag;
int startHiddenFlag = 0;
int countHidden = 0;
/**************nc test***********/

/* Advanced Page Mapping FTL:
 * - Block Dirty Table: LOG_BLOCK 0, cache all
 * - ROOT Table: LOG_BLOCK 1, cache all. point to journal blocks.
 * - Page Mapping Table: LOG_BLOCK 2~N, cache x pages with LRU algo.
 * - DATA Journal: commit
 * - Init: read BDT, ROOT, PMT, Journal info, ...
 * - Reclaim
 * - Meta Data Page: in last page in PMT blocks and data blocks.
 * - choose journal block on erase and write, according to die index
 *
 * TODO: advanced features:
 * - sanitizing
 * - bg erase
 * - check wp/trim, ...
 */

STATUS FTL_Format() {
  STATUS ret;
    
  ret = UBI_Format();
  if (ret == STATUS_SUCCESS) {
    ret = UBI_Init();
  }

  if (ret == STATUS_SUCCESS) {
    ret = DATA_Format();
  }

  if (ret == STATUS_SUCCESS) {
    ret = HDI_Format();
  }

  if (ret == STATUS_SUCCESS) {
    ret = PMT_Format();
  }

  if (ret == STATUS_SUCCESS) {
    ret = BDT_Format();
  }

  if (ret == STATUS_SUCCESS) {
    ret = ROOT_Format();
  }

  return ret;
}

STATUS FTL_Init() {
  STATUS ret;

  ret = UBI_Init();
  if (ret == STATUS_SUCCESS) {
    /* scan tables on UBI, and copy to RAM */
    ret = ROOT_Init();
  }

  if (ret == STATUS_SUCCESS) {
    ret = BDT_Init();
  }

  if (ret == STATUS_SUCCESS) {
    ret = PMT_Init();
  }

  if (ret == STATUS_SUCCESS) {
    ret = HDI_Init();
  }

  if (ret == STATUS_SUCCESS) {
    ret = DATA_Replay(root_table.hot_journal);
  }

  if (ret == STATUS_SUCCESS) {
    ret = DATA_Replay(root_table.cold_journal);
  }

  if (ret == STATUS_SUCCESS) {
    /* handle reclaim PLR: start reclaim again. Some data should
     * be written in the same place, so just rewrite same data in the
     * same page regardless this page is written or not. */

    /* check if hot journal blocks are full */
    if (DATA_IsFull(TRUE) == TRUE) {
      ret = DATA_Reclaim(TRUE);
      if (ret == STATUS_SUCCESS) {
        ret = DATA_Commit();
      }
    }

    /* check if cold journal blocks are full */
    if (DATA_IsFull(FALSE) == TRUE) {
      ret = DATA_Reclaim(FALSE);
      if (ret == STATUS_SUCCESS) {
        ret = DATA_Commit();
      }
    }
  }

  return ret;
}

STATUS FTL_Write(PGADDR addr, void* buffer) {
  STATUS ret;
  BOOL is_hot = HDI_IsHotPage(addr);
  /****************nc test***********/
//  if(addr == 213828) {
//    countHidden++;
//    if(countHidden == 2) {
//      hiddenFlag = 1;
//      uart_printf("enter hidden volume in FTL\n");
//    }
//  }
//  if(printSnap == 1) {
//    int secretOffset = 0;
//    //uart_printf("write: addr is %d\n", addr);
//    char dest[4]={0};
//    if(addr == 213828) {
////      uart_printf("write: content is %s\n",buffer);
////      strncpy(dest, ((char*)buffer) + 10,3);
////      uart_printf("dest is %s\n",dest);
////      secretOffset = atoi(dest);
////      uart_printf("secret offset is %d\n", secretOffset);
//      uart_printf("offset is %d\n",tempAddr);
//      
//    }
//  }
  /***************nc test***********/
    
  ret = DATA_Write(addr, buffer, is_hot);
  if (ret == STATUS_SUCCESS) {
    if(hiddenFlag == 0) { //************nc test***********: hidden flag is 0, which mean we are in public mode, so we should do gc
      if (DATA_IsFull(is_hot) == TRUE) {
        ret = DATA_Reclaim(is_hot);
        if (ret == STATUS_SUCCESS) {
          ret = DATA_Commit();
        }
      }
   }
  }
  return ret;
}

STATUS FTL_Read(PGADDR addr, void* buffer) {
  LOG_BLOCK block;
  PAGE_OFF page;
  STATUS ret;
  /****************nc test***********/
  static int countSector = 1;
  static int tempPage = 213828;
  static int countFlag = 0;
  if(printSnap == 1) {
     uart_printf("page address is %d\n",addr);
  }
  if(startHiddenFlag == 1) {
    if(addr == tempPage + 1) {
      countFlag = 1;
      tempPage = addr;
      countSector++;
    }
    else {
      startHiddenFlag = 0;
      tempPage = 213828;
      countFlag = 2;
    }
  }
  
  if(countFlag == 2) {
    if(countSector == 6) {
        hiddenFlag = 1;
        countSector = 1;
        uart_printf("enter hidden volume in FTL\n");
      }
    else if(countSector == 4) {
      hiddenFlag = 0;
      countSector = 1;
      uart_printf("exit hidden volume in FTL\n");
    }
    countFlag = 0;
  }
    if(addr == 213828) { //100 M 213828

      startHiddenFlag = 1;
  }
//  if(printSnap == 1) {
//    //uart_printf("read: addr is %d,\n", addr);
//    if(addr == 200000) {
//      uart_printf("read: content is %s\n",buffer);
//    }
//  }
  /***************nc test***********/
  ret = PMT_Search(addr, &block, &page);
  if (ret == STATUS_SUCCESS) {
    ret = UBI_Read(block, page, buffer, NULL);
  }

  return ret;
}

STATUS FTL_Trim(PGADDR start, PGADDR end) {
  PGADDR addr;
  STATUS ret = STATUS_SUCCESS;

  for (addr = start; addr <= end; addr++) {
    ret = FTL_Write(addr, NULL);
    if (ret != STATUS_SUCCESS) {
      break;
    }
  }

  return ret;
}

STATUS FTL_SetWP(PGADDR laddr, BOOL enabled) {
  return STATUS_FAILURE;
}

BOOL FTL_CheckWP(PGADDR laddr) {
  return FALSE;
}

STATUS FTL_BgTasks() {
  return STATUS_SUCCESS;
}

PGADDR FTL_Capacity() {
  LOG_BLOCK block;

  block = UBI_Capacity;//3989
  block -= JOURNAL_BLOCK_COUNT; /* data hot journal *///1
  block -= JOURNAL_BLOCK_COUNT; /* data cold journal *///1
  block -= JOURNAL_BLOCK_COUNT; /* data reclaim journal *///1
  block -= PMT_BLOCK_COUNT; /* pmt blocks *///40
  block -= 2; /* bdt blocks */
  block -= 2; /* root blocks */
  block -= 2; /* hdi reserved */
  block -= block / 100 * OVER_PROVISION_RATE; /* over provision */
  /*
  uart_printf("%s: UBI_Capacity=%d\r\n",__func__,UBI_Capacity);
  uart_printf("%s: actual user capacity: block=%d\r\n",__func__,block);//3823
*/
  /* last page in every block is reserved for meta data collection */
  return block * (PAGE_PER_PHY_BLOCK - 1);//471
}

STATUS FTL_Flush() {
  STATUS ret;

  ret = DATA_Commit();
  if (ret == STATUS_SUCCESS) {
    ret = UBI_Flush();
  }

  if (ret == STATUS_SUCCESS) {
    ret = UBI_SWL();
  }

  return ret;
}
